@@ -3,14 +3,18 @@ import os
 import sys
 import pyuv
 
+if sys.version_info >= (3, 0):
+    LINESEP = os.linesep.encode()
+else:
+    LINESEP = os.linesep
 
 def on_pipe_read(handle, data, error):
     data = data.strip()
-    if data == "exit":
+    if data == b"exit":
         pipe_stdin.close()
         pipe_stdout.close()
     else:
-        pipe_stdout.write([data, os.linesep])
+        pipe_stdout.write([data, LINESEP])
 
 
 loop = pyuv.Loop.default_loop()
