@@ -2,6 +2,107 @@
 #include "nameser.h"
 
 
+static PyTypeObject DNSHostResultType = {0, 0, 0, 0, 0, 0};
+
+static PyStructSequence_Field dns_host_result_fields[] = {
+    {"name", ""},
+    {"aliases", ""},
+    {"addresses", ""},
+    {NULL}
+};
+
+static PyStructSequence_Desc dns_host_result_desc = {
+    "dns_host_result",
+    NULL,
+    dns_host_result_fields,
+    3
+};
+
+static PyTypeObject DNSNameinfoResultType = {0, 0, 0, 0, 0, 0};
+
+static PyStructSequence_Field dns_nameinfo_result_fields[] = {
+    {"node", ""},
+    {"service", ""},
+    {NULL}
+};
+
+static PyStructSequence_Desc dns_nameinfo_result_desc = {
+    "dns_nameinfo_result",
+    NULL,
+    dns_nameinfo_result_fields,
+    2
+};
+
+static PyTypeObject DNSAddrinfoResultType = {0, 0, 0, 0, 0, 0};
+
+static PyStructSequence_Field dns_addrinfo_result_fields[] = {
+    {"family", ""},
+    {"socktype", ""},
+    {"proto", ""},
+    {"canonname", ""},
+    {"sockaddr", ""},
+    {NULL}
+};
+
+static PyStructSequence_Desc dns_addrinfo_result_desc = {
+    "dns_addrinfo_result",
+    NULL,
+    dns_addrinfo_result_fields,
+    5
+};
+
+static PyTypeObject DNSQueryMXResultType = {0, 0, 0, 0, 0, 0};
+
+static PyStructSequence_Field dns_query_mx_result_fields[] = {
+    {"host", ""},
+    {"priority", ""},
+    {NULL}
+};
+
+static PyStructSequence_Desc dns_query_mx_result_desc = {
+    "dns_query_mx_result",
+    NULL,
+    dns_query_mx_result_fields,
+    2
+};
+
+static PyTypeObject DNSQuerySRVResultType = {0, 0, 0, 0, 0, 0};
+
+static PyStructSequence_Field dns_query_srv_result_fields[] = {
+    {"host", ""},
+    {"port", ""},
+    {"priority", ""},
+    {"weight", ""},
+    {NULL}
+};
+
+static PyStructSequence_Desc dns_query_srv_result_desc = {
+    "dns_query_srv_result",
+    NULL,
+    dns_query_srv_result_fields,
+    4
+};
+
+static PyTypeObject DNSQueryNAPTRResultType = {0, 0, 0, 0, 0, 0};
+
+static PyStructSequence_Field dns_query_naptr_result_fields[] = {
+    {"order", ""},
+    {"preference", ""},
+    {"flags", ""},
+    {"service", ""},
+    {"regex", ""},
+    {"replacement", ""},
+    {NULL}
+};
+
+static PyStructSequence_Desc dns_query_naptr_result_desc = {
+    "dns_query_naptr_result",
+    NULL,
+    dns_query_naptr_result_fields,
+    6
+};
+
+
 static PyObject* PyExc_DNSError;
 
 
@@ -40,7 +141,7 @@ host_cb(void *arg, int status, int timeouts, struct hostent *hostent)
 
     dns_aliases = PyList_New(0);
     dns_addrlist = PyList_New(0);
-    dns_result = PyTuple_New(3);
+    dns_result = PyStructSequence_New(&DNSHostResultType);
 
     if (!(dns_aliases && dns_addrlist && dns_result)) {
         PyErr_NoMemory();
@@ -82,9 +183,9 @@ host_cb(void *arg, int status, int timeouts, struct hostent *hostent)
     }
     dns_name = PYUVString_FromString(hostent->h_name);
 
-    PyTuple_SET_ITEM(dns_result, 0, dns_name);
-    PyTuple_SET_ITEM(dns_result, 1, dns_aliases);
-    PyTuple_SET_ITEM(dns_result, 2, dns_addrlist);
+    PyStructSequence_SET_ITEM(dns_result, 0, dns_name);
+    PyStructSequence_SET_ITEM(dns_result, 1, dns_aliases);
+    PyStructSequence_SET_ITEM(dns_result, 2, dns_addrlist);
     errorno = Py_None;
     Py_INCREF(Py_None);
 
@@ -130,7 +231,7 @@ nameinfo_cb(void *arg, int status, int timeouts, char *node, char *service)
         goto callback;
     }
 
-    dns_result = PyTuple_New(2);
+    dns_result = PyStructSequence_New(&DNSNameinfoResultType);
     if (!dns_result) {
         PyErr_NoMemory();
         PyErr_WriteUnraisable(Py_None);
@@ -148,8 +249,8 @@ nameinfo_cb(void *arg, int status, int timeouts, char *node, char *service)
         Py_INCREF(Py_None);
     }
 
-    PyTuple_SET_ITEM(dns_result, 0, dns_node);
-    PyTuple_SET_ITEM(dns_result, 1, dns_service);
+    PyStructSequence_SET_ITEM(dns_result, 0, dns_node);
+    PyStructSequence_SET_ITEM(dns_result, 1, dns_service);
     errorno = Py_None;
     Py_INCREF(Py_None);
 
@@ -251,13 +352,17 @@ getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
             break;
         }
 
-        item = Py_BuildValue("iiisO", ptr->ai_family, ptr->ai_socktype, ptr->ai_protocol, ptr->ai_canonname ? ptr->ai_canonname : "", addr);
-        Py_DECREF(addr);
+        item = PyStructSequence_New(&DNSAddrinfoResultType);
         if (!item) {
             PyErr_NoMemory();
             PyErr_WriteUnraisable(callback);
             break;
         }
+        PyStructSequence_SET_ITEM(item, 0, PyInt_FromLong((long)ptr->ai_family));
+        PyStructSequence_SET_ITEM(item, 1, PyInt_FromLong((long)ptr->ai_socktype));
+        PyStructSequence_SET_ITEM(item, 2, PyInt_FromLong((long)ptr->ai_protocol));
+        PyStructSequence_SET_ITEM(item, 3, PYUVString_FromString(ptr->ai_canonname ? ptr->ai_canonname : ""));
+        PyStructSequence_SET_ITEM(item, 4, addr);
 
         PyList_Append(dns_result, item);
         Py_DECREF(item);
@@ -553,12 +658,12 @@ query_mx_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int a
     }
 
     for (mx_ptr = mx_reply; mx_ptr != NULL; mx_ptr = mx_ptr->next) {
-        tmp = PyTuple_New(2);
+        tmp = PyStructSequence_New(&DNSQueryMXResultType);
         if (tmp == NULL) {
             break;
         }
-        PyTuple_SET_ITEM(tmp, 0, PYUVString_FromString(mx_ptr->host));
-        PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)mx_ptr->priority));
+        PyStructSequence_SET_ITEM(tmp, 0, PYUVString_FromString(mx_ptr->host));
+        PyStructSequence_SET_ITEM(tmp, 1, PyInt_FromLong((long)mx_ptr->priority));
         PyList_Append(dns_result, tmp);
         Py_DECREF(tmp);
     }
@@ -778,14 +883,14 @@ query_srv_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     }
 
     for (srv_ptr = srv_reply; srv_ptr != NULL; srv_ptr = srv_ptr->next) {
-        tmp = PyTuple_New(4);
+        tmp = PyStructSequence_New(&DNSQuerySRVResultType);
         if (tmp == NULL) {
             break;
         }
-        PyTuple_SET_ITEM(tmp, 0, PYUVString_FromString(srv_ptr->host));
-        PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)srv_ptr->port));
-        PyTuple_SET_ITEM(tmp, 2, PyInt_FromLong((long)srv_ptr->priority));
-        PyTuple_SET_ITEM(tmp, 3, PyInt_FromLong((long)srv_ptr->weight));
+        PyStructSequence_SET_ITEM(tmp, 0, PYUVString_FromString(srv_ptr->host));
+        PyStructSequence_SET_ITEM(tmp, 1, PyInt_FromLong((long)srv_ptr->port));
+        PyStructSequence_SET_ITEM(tmp, 2, PyInt_FromLong((long)srv_ptr->priority));
+        PyStructSequence_SET_ITEM(tmp, 3, PyInt_FromLong((long)srv_ptr->weight));
         PyList_Append(dns_result, tmp);
         Py_DECREF(tmp);
     }
@@ -856,16 +961,16 @@ query_naptr_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, in
     }
 
     for (naptr_ptr = naptr_reply; naptr_ptr != NULL; naptr_ptr = naptr_ptr->next) {
-        tmp = PyTuple_New(6);
+        tmp = PyStructSequence_New(&DNSQueryNAPTRResultType);
         if (tmp == NULL) {
             break;
         }
-        PyTuple_SET_ITEM(tmp, 0, PyInt_FromLong((long)naptr_ptr->order));
-        PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)naptr_ptr->preference));
-        PyTuple_SET_ITEM(tmp, 2, PYUVString_FromString((char *)naptr_ptr->flags));
-        PyTuple_SET_ITEM(tmp, 3, PYUVString_FromString((char *)naptr_ptr->service));
-        PyTuple_SET_ITEM(tmp, 4, PYUVString_FromString((char *)naptr_ptr->regexp));
-        PyTuple_SET_ITEM(tmp, 5, PYUVString_FromString(naptr_ptr->replacement));
+        PyStructSequence_SET_ITEM(tmp, 0, PyInt_FromLong((long)naptr_ptr->order));
+        PyStructSequence_SET_ITEM(tmp, 1, PyInt_FromLong((long)naptr_ptr->preference));
+        PyStructSequence_SET_ITEM(tmp, 2, PYUVString_FromString((char *)naptr_ptr->flags));
+        PyStructSequence_SET_ITEM(tmp, 3, PYUVString_FromString((char *)naptr_ptr->service));
+        PyStructSequence_SET_ITEM(tmp, 4, PYUVString_FromString((char *)naptr_ptr->regexp));
+        PyStructSequence_SET_ITEM(tmp, 5, PYUVString_FromString(naptr_ptr->replacement));
         PyList_Append(dns_result, tmp);
         Py_DECREF(tmp);
     }
@@ -1098,13 +1203,14 @@ error:
 
 
 static PyObject *
-DNSResolver_func_query_a(DNSResolver *self, PyObject *args)
+DNSResolver_func_query(DNSResolver *self, PyObject *args)
 {
     char *name;
+    int query_type;
     ares_cb_data_t *cb_data;
     PyObject *callback;
 
-    if (!PyArg_ParseTuple(args, "sO:query_a", &name, &callback)) {
+    if (!PyArg_ParseTuple(args, "isO:query_a", &query_type, &name, &callback)) {
         return NULL;
     }
 
@@ -1122,225 +1228,64 @@ DNSResolver_func_query_a(DNSResolver *self, PyObject *args)
     cb_data->resolver = self;
     cb_data->cb = callback;
 
-    ares_query(self->channel, name, C_IN, T_A, &query_a_cb, (void *)cb_data);
+    switch (query_type) {
+        case T_A:
+        {
+            ares_query(self->channel, name, C_IN, T_A, &query_a_cb, (void *)cb_data);
+            break;
+        }
 
-    Py_RETURN_NONE;
-}
+        case T_AAAA:
+        {
+            ares_query(self->channel, name, C_IN, T_AAAA, &query_aaaa_cb, (void *)cb_data);
+            break;
+        }
 
+        case T_CNAME:
+        {
+            ares_query(self->channel, name, C_IN, T_CNAME, &query_cname_cb, (void *)cb_data);
+            break;
+        }
 
-static PyObject *
-DNSResolver_func_query_aaaa(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
+        case T_MX:
+        {
+            ares_query(self->channel, name, C_IN, T_MX, &query_mx_cb, (void *)cb_data);
+            break;
+        }
 
-    if (!PyArg_ParseTuple(args, "sO:query_aaaa", &name, &callback)) {
-        return NULL;
+        case T_NAPTR:
+        {
+            ares_query(self->channel, name, C_IN, T_NAPTR, &query_naptr_cb, (void *)cb_data);
+            break;
+        }
+
+        case T_NS:
+        {
+            ares_query(self->channel, name, C_IN, T_NS, &query_ns_cb, (void *)cb_data);
+            break;
+        }
+
+        case T_SRV:
+        {
+            ares_query(self->channel, name, C_IN, T_SRV, &query_srv_cb, (void *)cb_data);
+            break;
+        }
+
+        case T_TXT:
+        {
+            ares_query(self->channel, name, C_IN, T_TXT, &query_txt_cb, (void *)cb_data);
+            break;
+        }
+
+        default:
+        {
+            Py_DECREF(callback);
+            PyMem_Free(cb_data);
+            PyErr_SetString(PyExc_DNSError, "invalid query type specified");
+            return NULL;
+        }
     }
 
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_AAAA, &query_aaaa_cb, (void *)cb_data);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-DNSResolver_func_query_cname(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "sO:query_cname", &name, &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_CNAME, &query_cname_cb, (void *)cb_data);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-DNSResolver_func_query_mx(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "sO:query_mx", &name, &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_MX, &query_mx_cb, (void *)cb_data);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-DNSResolver_func_query_ns(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "sO:query_ns", &name, &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_NS, &query_ns_cb, (void *)cb_data);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-DNSResolver_func_query_txt(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "sO:query_txt", &name, &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_TXT, &query_txt_cb, (void *)cb_data);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-DNSResolver_func_query_srv(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "sO:query_srv", &name, &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_SRV, &query_srv_cb, (void *)cb_data);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-DNSResolver_func_query_naptr(DNSResolver *self, PyObject *args)
-{
-    char *name;
-    ares_cb_data_t *cb_data;
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "sO:query_naptr", &name, &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
-    if (!cb_data) {
-        return PyErr_NoMemory();
-    }
-
-    Py_INCREF(callback);
-    cb_data->resolver = self;
-    cb_data->cb = callback;
-
-    ares_query(self->channel, name, C_IN, T_NAPTR, &query_naptr_cb, (void *)cb_data);
-
     Py_RETURN_NONE;
 }
 
@@ -1566,14 +1511,7 @@ DNSResolver_tp_methods[] = {
     { "gethostbyaddr", (PyCFunction)DNSResolver_func_gethostbyaddr, METH_VARARGS|METH_KEYWORDS, "Gethostbyaddr" },
     { "getnameinfo", (PyCFunction)DNSResolver_func_getnameinfo, METH_VARARGS|METH_KEYWORDS, "Getnameinfo" },
     { "getaddrinfo", (PyCFunction)DNSResolver_func_getaddrinfo, METH_VARARGS|METH_KEYWORDS, "Getaddrinfo" },
-    { "query_a", (PyCFunction)DNSResolver_func_query_a, METH_VARARGS, "Run a DNS query of type A" },
-    { "query_aaaa", (PyCFunction)DNSResolver_func_query_aaaa, METH_VARARGS, "Run a DNS query of type AAAA" },
-    { "query_cname", (PyCFunction)DNSResolver_func_query_cname, METH_VARARGS, "Run a DNS query of type CNAME" },
-    { "query_mx", (PyCFunction)DNSResolver_func_query_mx, METH_VARARGS, "Run a DNS query of type MX" },
-    { "query_ns", (PyCFunction)DNSResolver_func_query_ns, METH_VARARGS, "Run a DNS query of type NS" },
-    { "query_txt", (PyCFunction)DNSResolver_func_query_txt, METH_VARARGS, "Run a DNS query of type TXT" },
-    { "query_srv", (PyCFunction)DNSResolver_func_query_srv, METH_VARARGS, "Run a DNS query of type SRV" },
-    { "query_naptr", (PyCFunction)DNSResolver_func_query_naptr, METH_VARARGS, "Run a DNS query of type NAPTR" },
+    { "query", (PyCFunction)DNSResolver_func_query, METH_VARARGS, "Run a DNS query of the specified type" },
     { NULL }
 };
 
@@ -1672,6 +1610,33 @@ init_dns(void)
     PyModule_AddIntMacro(module, ARES_NI_IDN_ALLOW_UNASSIGNED);
     PyModule_AddIntMacro(module, ARES_NI_IDN_USE_STD3_ASCII_RULES);
 
+#define QUERY_TYPE_A        T_A
+#define QUERY_TYPE_AAAA     T_AAAA
+#define QUERY_TYPE_CNAME    T_CNAME
+#define QUERY_TYPE_MX       T_MX
+#define QUERY_TYPE_NAPTR    T_NAPTR
+#define QUERY_TYPE_NS       T_NS
+#define QUERY_TYPE_SRV      T_SRV
+#define QUERY_TYPE_TXT      T_TXT
+
+    PyModule_AddIntMacro(module, QUERY_TYPE_A);
+    PyModule_AddIntMacro(module, QUERY_TYPE_AAAA);
+    PyModule_AddIntMacro(module, QUERY_TYPE_CNAME);
+    PyModule_AddIntMacro(module, QUERY_TYPE_MX);
+    PyModule_AddIntMacro(module, QUERY_TYPE_NAPTR);
+    PyModule_AddIntMacro(module, QUERY_TYPE_NS);
+    PyModule_AddIntMacro(module, QUERY_TYPE_SRV);
+    PyModule_AddIntMacro(module, QUERY_TYPE_TXT);
+
+#undef QUERY_TYPE_A
+#undef QUERY_TYPE_AAAA
+#undef QUERY_TYPE_CNAME
+#undef QUERY_TYPE_MX
+#undef QUERY_TYPE_NAPTR
+#undef QUERY_TYPE_NS
+#undef QUERY_TYPE_SRV
+#undef QUERY_TYPE_TXT
+
     PyUVModule_AddType(module, "DNSResolver", &DNSResolverType);
 
     return module;
