@@ -1,7 +1,6 @@
 
 #include "pyuv.h"
 
-#include "errno.c"
 #include "helper.c"
 
 #include "loop.c"
@@ -21,7 +20,7 @@
 #include "threadpool.c"
 #include "process.c"
 #include "util.c"
-
+#include "errno.c"
 #include "error.c"
 
 #define LIBUV_VERSION UV_VERSION_MAJOR.UV_VERSION_MINOR-LIBUV_REVISION
@@ -112,25 +111,37 @@ init_pyuv(void)
     PyUVModule_AddType(pyuv, "ThreadPool", &ThreadPoolType);
     PyUVModule_AddType(pyuv, "Process", &ProcessType);
 
+    /* PyStructSequence types */
+    if (LoopCountersResultType.tp_name == 0)
+        PyStructSequence_InitType(&LoopCountersResultType, &loop_counters_result_desc);
+    if (DNSHostResultType.tp_name == 0)
+        PyStructSequence_InitType(&DNSHostResultType, &dns_host_result_desc);
+    if (DNSNameinfoResultType.tp_name == 0)
+        PyStructSequence_InitType(&DNSNameinfoResultType, &dns_nameinfo_result_desc);
+    if (DNSAddrinfoResultType.tp_name == 0)
+        PyStructSequence_InitType(&DNSAddrinfoResultType, &dns_addrinfo_result_desc);
+    if (DNSQueryMXResultType.tp_name == 0)
+        PyStructSequence_InitType(&DNSQueryMXResultType, &dns_query_mx_result_desc);
+    if (DNSQuerySRVResultType.tp_name == 0)
+        PyStructSequence_InitType(&DNSQuerySRVResultType, &dns_query_srv_result_desc);
+    if (DNSQueryNAPTRResultType.tp_name == 0)
+        PyStructSequence_InitType(&DNSQueryNAPTRResultType, &dns_query_naptr_result_desc);
+    if (StatResultType.tp_name == 0)
+        PyStructSequence_InitType(&StatResultType, &stat_result_desc);
+
     /* Constants */
     PyModule_AddIntMacro(pyuv, UV_JOIN_GROUP);
     PyModule_AddIntMacro(pyuv, UV_LEAVE_GROUP);
+    PyModule_AddIntMacro(pyuv, UV_PROCESS_SETUID);
+    PyModule_AddIntMacro(pyuv, UV_PROCESS_SETGID);
+    PyModule_AddIntMacro(pyuv, UV_PROCESS_WINDOWS_VERBATIM_ARGUMENTS);
 
+    /* Handle types */
     PyModule_AddIntMacro(pyuv, UV_UNKNOWN_HANDLE);
-    PyModule_AddIntMacro(pyuv, UV_TCP);
-    PyModule_AddIntMacro(pyuv, UV_UDP);
-    PyModule_AddIntMacro(pyuv, UV_NAMED_PIPE);
-    PyModule_AddIntMacro(pyuv, UV_TTY);
     PyModule_AddIntMacro(pyuv, UV_FILE);
-    PyModule_AddIntMacro(pyuv, UV_TIMER);
-    PyModule_AddIntMacro(pyuv, UV_PREPARE);
-    PyModule_AddIntMacro(pyuv, UV_CHECK);
-    PyModule_AddIntMacro(pyuv, UV_IDLE);
-    PyModule_AddIntMacro(pyuv, UV_ASYNC);
-    PyModule_AddIntMacro(pyuv, UV_ARES_TASK);
-    PyModule_AddIntMacro(pyuv, UV_ARES_EVENT);
-    PyModule_AddIntMacro(pyuv, UV_PROCESS);
-    PyModule_AddIntMacro(pyuv, UV_FS_EVENT);
+#define XX(uc, lc) PyModule_AddIntMacro(pyuv, UV_##uc);
+    UV_HANDLE_TYPE_MAP(XX)
+#undef XX
 
     /* Module version (the MODULE_VERSION macro is defined by setup.py) */
     PyModule_AddStringConstant(pyuv, "__version__", __MSTR(MODULE_VERSION));
