@@ -32,6 +32,7 @@ on_tcp_connection(uv_stream_t* server, int status)
         PyErr_WriteUnraisable(self->on_new_connection_cb);
     }
     Py_XDECREF(result);
+    Py_DECREF(py_errorno);
 
     Py_DECREF(self);
     PyGILState_Release(gstate);
@@ -72,6 +73,7 @@ on_tcp_client_connection(uv_connect_t *req, int status)
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(py_errorno);
 
     Py_DECREF(callback);
     PyMem_Free(req_data);
@@ -216,7 +218,7 @@ TCP_func_connect(TCP *self, PyObject *args)
     struct in6_addr addr6;
     uv_connect_t *connect_req = NULL;
     iostream_req_data_t *req_data = NULL;
-    PyObject *connect_address, *callback;
+    PyObject *callback;
 
     IOStream *base = (IOStream *)self;
 
@@ -225,11 +227,7 @@ TCP_func_connect(TCP *self, PyObject *args)
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(args, "OO:connect", &connect_address, &callback)) {
-        return NULL;
-    }
-
-    if (!PyArg_ParseTuple(connect_address, "si", &connect_ip, &connect_port)) {
+    if (!PyArg_ParseTuple(args, "(si)O:connect", &connect_ip, &connect_port, &callback)) {
         return NULL;
     }
 
