@@ -6,12 +6,14 @@ static void
 on_timer_close(uv_handle_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
-    ASSERT(handle);
-    Timer *self = (Timer *)handle->data;
-    ASSERT(self);
-
+    Timer *self;
     PyObject *result;
 
+    ASSERT(handle);
+
+    self = (Timer *)handle->data;
+    ASSERT(self);
+
     if (self->on_close_cb != Py_None) {
         result = PyObject_CallFunctionObjArgs(self->on_close_cb, self, NULL);
         if (result == NULL) {
@@ -45,15 +47,17 @@ static void
 on_timer_callback(uv_timer_t *timer, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    Timer *self;
+    PyObject *result;
+
     ASSERT(timer);
     ASSERT(status == 0);
 
-    Timer *self = (Timer *)(timer->data);
+    self = (Timer *)(timer->data);
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    PyObject *result;
     result = PyObject_CallFunctionObjArgs(self->callback, self, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(self->callback);
@@ -68,14 +72,14 @@ on_timer_callback(uv_timer_t *timer, int status)
 static PyObject *
 Timer_func_start(Timer *self, PyObject *args, PyObject *kwargs)
 {
-    int r = 0;
-    double timeout;
-    double repeat;
-    PyObject *tmp = NULL;
-    PyObject *callback;
+    int r;
+    double timeout, repeat;
+    PyObject *tmp, *callback;
 
     static char *kwlist[] = {"callback", "timeout", "repeat", NULL};
 
+    tmp = NULL;
+
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_TimerError, "Timer is closed");
         return NULL;
@@ -118,12 +122,14 @@ Timer_func_start(Timer *self, PyObject *args, PyObject *kwargs)
 static PyObject *
 Timer_func_stop(Timer *self)
 {
+    int r;
+
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_TimerError, "Timer is closed");
         return NULL;
     }
 
-    int r = uv_timer_stop(self->uv_handle);
+    r = uv_timer_stop(self->uv_handle);
     if (r != 0) {
         raise_uv_exception(self->loop, PyExc_TimerError);
         return NULL;
@@ -136,12 +142,14 @@ Timer_func_stop(Timer *self)
 static PyObject *
 Timer_func_again(Timer *self)
 {
+    int r;
+
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_TimerError, "Timer is closed");
         return NULL;
     }
 
-    int r = uv_timer_again(self->uv_handle);
+    r = uv_timer_again(self->uv_handle);
     if (r != 0) {
         raise_uv_exception(self->loop, PyExc_TimerError);
         return NULL;
@@ -186,6 +194,7 @@ Timer_func_close(Timer *self, PyObject *args)
 static PyObject *
 Timer_active_get(Timer *self, void *closure)
 {
+    UNUSED_ARG(closure);
     if (!self->uv_handle) {
         Py_RETURN_FALSE;
     } else {
@@ -197,11 +206,11 @@ Timer_active_get(Timer *self, void *closure)
 static PyObject *
 Timer_repeat_get(Timer *self, void *closure)
 {
+    UNUSED_ARG(closure);
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_TimerError, "Timer is closed");
         return NULL;
     }
-
     return PyFloat_FromDouble(uv_timer_get_repeat(self->uv_handle)/1000.0);
 }
 
@@ -211,6 +220,8 @@ Timer_repeat_set(Timer *self, PyObject *value, void *closure)
 {
     double repeat;
 
+    UNUSED_ARG(closure);
+
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_TimerError, "Timer is closed");
         return -1;
@@ -240,10 +251,12 @@ Timer_repeat_set(Timer *self, PyObject *value, void *closure)
 static int
 Timer_tp_init(Timer *self, PyObject *args, PyObject *kwargs)
 {
-    int r = 0;
+    int r;
+    uv_timer_t *uv_timer = NULL;
     Loop *loop;
     PyObject *tmp = NULL;
-    uv_timer_t *uv_timer = NULL;
+
+    UNUSED_ARG(kwargs);
 
     if (self->uv_handle) {
         PyErr_SetString(PyExc_TimerError, "Object already initialized");
