@@ -1,11 +1,11 @@
 
 import threading
 
-import common
+from common import unittest2
 import pyuv
 
 
-class AsyncTest(common.UVTestCase):
+class AsyncTest(unittest2.TestCase):
 
     def test_async1(self):
         self.async_cb_called = 0
@@ -26,7 +26,5 @@ class AsyncTest(common.UVTestCase):
 
 
 if __name__ == '__main__':
-    import unittest
-    tests = unittest.TestSuite(common.suites)
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    unittest2.main(verbosity=2)
 
